@@ -647,20 +647,21 @@ set_element (fd_set *set, SCM *ports_ready, SCM element, int pos)
   else
     {
       int use_buf = 0;
+      size_t cur;
 
       element = SCM_COERCE_OUTPORT (element);
       SCM_ASSERT (SCM_OPFPORTP (element), element, pos, "select");
       if (pos == SCM_ARG1)
 	{
 	  /* Check whether port has input buffered.  */
-	  if (scm_port_buffer_can_take (SCM_PORT (element)->read_buf) > 0)
+	  if (scm_port_buffer_can_take (SCM_PORT (element)->read_buf, &cur) > 0)
 	    use_buf = 1;
 	}
       else if (pos == SCM_ARG2)
 	{
 	  /* Check whether port's output buffer has room.  > 1 since
              writing the last byte in the buffer causes flush.  */
-	  if (scm_port_buffer_can_put (SCM_PORT (element)->write_buf) > 1)
+	  if (scm_port_buffer_can_put (SCM_PORT (element)->write_buf, &cur) > 1)
 	    use_buf = 1;
 	}
       fd = use_buf ? -1 : SCM_FPORT_FDES (element);
@@ -1508,7 +1509,7 @@ SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
 
   scm_dynwind_end ();
 
-  port = scm_i_fdes_to_port (rv, mode_bits, tmpl);
+  port = scm_i_fdes_to_port (rv, mode_bits, tmpl, 0);
   if (is_binary)
     /* Use the binary-friendly ISO-8859-1 encoding. */
     scm_i_set_port_encoding_x (port, NULL);
