@@ -367,7 +367,12 @@ SCM_DEFINE (scm_inet_pton, "inet-pton", 2, 0, 0,
 
 SCM_SYMBOL (sym_socket, "socket");
 
-#define SCM_SOCK_FD_TO_PORT(fd) scm_fdes_to_port (fd, "r+0", sym_socket)
+static SCM
+scm_socket_fd_to_port (int fd)
+{
+  return scm_i_fdes_to_port (fd, scm_mode_bits ("r+0"), sym_socket,
+                             SCM_FPORT_OPTION_NOT_SEEKABLE);
+}
 
 SCM_DEFINE (scm_socket, "socket", 3, 0, 0,
             (SCM family, SCM style, SCM proto),
@@ -391,7 +396,7 @@ SCM_DEFINE (scm_socket, "socket", 3, 0, 0,
 	       scm_to_int (proto));
   if (fd == -1)
     SCM_SYSERROR;
-  return SCM_SOCK_FD_TO_PORT (fd);
+  return scm_socket_fd_to_port (fd);
 }
 #undef FUNC_NAME
 
@@ -413,7 +418,8 @@ SCM_DEFINE (scm_socketpair, "socketpair", 3, 0, 0,
   if (socketpair (fam, scm_to_int (style), scm_to_int (proto), fd) == -1)
     SCM_SYSERROR;
 
-  return scm_cons (SCM_SOCK_FD_TO_PORT (fd[0]), SCM_SOCK_FD_TO_PORT (fd[1]));
+  return scm_cons (scm_socket_fd_to_port (fd[0]),
+                   scm_socket_fd_to_port (fd[1]));
 }
 #undef FUNC_NAME
 #endif
@@ -1237,8 +1243,8 @@ SCM_DEFINE (scm_make_socket_address, "make-socket-address", 2, 0, 1,
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_accept, "accept", 1, 0, 0, 
-            (SCM sock),
+SCM_DEFINE (scm_accept4, "accept", 1, 1, 0, 
+            (SCM sock, SCM flags),
 	    "Accept a connection on a bound, listening socket.  If there\n"
 	    "are no pending connections in the queue, there are two\n"
             "possibilities: if the socket has been configured as\n"
@@ -1250,10 +1256,11 @@ SCM_DEFINE (scm_accept, "accept", 1, 0, 0,
             "initiated the connection.\n\n"
 	    "@var{sock} does not become part of the\n"
 	    "connection and will continue to accept new requests.")
-#define FUNC_NAME s_scm_accept
+#define FUNC_NAME s_scm_accept4
 {
   int fd;
   int newfd;
+  int c_flags;
   SCM address;
   SCM newsock;
   socklen_t addr_size = MAX_ADDR_SIZE;
@@ -1261,22 +1268,30 @@ SCM_DEFINE (scm_accept, "accept", 1, 0, 0,
 
   sock = SCM_COERCE_OUTPORT (sock);
   SCM_VALIDATE_OPFPORT (1, sock);
+  c_flags = SCM_UNBNDP (flags) ? 0 : scm_to_int (flags);
+
   fd = SCM_FPORT_FDES (sock);
-  SCM_SYSCALL (newfd = accept (fd, (struct sockaddr *) &addr, &addr_size));
+  SCM_SYSCALL (newfd = accept4 (fd, (struct sockaddr *) &addr, &addr_size,
+                                c_flags));
   if (newfd == -1)
     {
       if (errno == EAGAIN || errno == EWOULDBLOCK)
         return SCM_BOOL_F;
       SCM_SYSERROR;
     }
-  newsock = SCM_SOCK_FD_TO_PORT (newfd);
-  address = _scm_from_sockaddr (&addr, addr_size,
-				FUNC_NAME);
+  newsock = scm_socket_fd_to_port (newfd);
+  address = _scm_from_sockaddr (&addr, addr_size, FUNC_NAME);
 
   return scm_cons (newsock, address);
 }
 #undef FUNC_NAME
 
+SCM
+scm_accept (SCM sock)
+{
+  return scm_accept4 (sock, SCM_UNDEFINED);
+}
+
 SCM_DEFINE (scm_getsockname, "getsockname", 1, 0, 0, 
             (SCM sock),
 	    "Return the address of @var{sock}, in the same form as the\n"
@@ -1638,6 +1653,11 @@ scm_init_socket ()
   scm_c_define ("SOCK_RDM", scm_from_int (SOCK_RDM));
 #endif
 
+  /* accept4 flags.  No ifdef as accept4 has a gnulib
+     implementation.  */
+  scm_c_define ("SOCK_CLOEXEC", scm_from_int (SOCK_CLOEXEC));
+  scm_c_define ("SOCK_NONBLOCK", scm_from_int (SOCK_NONBLOCK));
+
   /* setsockopt level.
 
      SOL_IP, SOL_TCP and SOL_UDP are defined on gnu/linux, but not on for
